package internalWindows;

import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;
import javax.imageio.*;
import javax.swing.*;

import org.simpleframework.xml.*;
import org.simpleframework.xml.core.*;
import org.simpleframework.xml.stream.*;

import java.io.*; // File operations

import mapObjects.*;

/**
 * This class represent a map editor frame.
 * @author Fredrik Stigsson
 *
 */
public class MapEditorFrame extends InternalFrame
{
	//*********************************************************************************
	// Class variables
	//*********************************************************************************
	private static final long serialVersionUID = -8186336718840236394L;
	
	//*********************************************************************************
	// Instance variables
	//*********************************************************************************
	private Map2D currentMap;
	private BufferedImage tileMapImage;
	private ImageTile[] imageTiles;
	private MapEditorPanel mapEditorPanel;
	private ImageTilePanel imageBlockPanel;
	private JScrollPane mapEditorScrollBar;
	private JScrollPane imageBlockScrollBar;
	private short selectedImage;
	
	//*********************************************************************************
	// Constructors
	//*********************************************************************************
	/**
	 * Create a new map editor frame
	 * @param currentMap
	 */
	public MapEditorFrame(Map2D currentMap, Dimension frameSize)
	{
		// Call the constructor in the base class
		super("Map editor", frameSize, new Point(0,0));

		// Set instance variables
		this.currentMap = currentMap;
		this.selectedImage = 0;
		
		// Create the ImageTiles list
		Dimension imgPanelDimension = CreateImageMap();

		// Add a map editor panel
		this.mapEditorPanel = new MapEditorPanel(null);
		this.mapEditorPanel.setPreferredSize(new Dimension(currentMap.mapWidth * currentMap.tileWidth, 
				currentMap.mapHeight * currentMap.tileHeight));
		
		// Add a image block panel
		this.imageBlockPanel = new ImageTilePanel(null);
		this.imageBlockPanel.setPreferredSize(new Dimension(imgPanelDimension.width, imgPanelDimension.height));
		
		// Set scroll bars for panels
		this.mapEditorScrollBar = new JScrollPane(this.mapEditorPanel,JScrollPane.VERTICAL_SCROLLBAR_ALWAYS,JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS);
		this.mapEditorScrollBar.getViewport().setScrollMode(JViewport.BACKINGSTORE_SCROLL_MODE);
		this.add(mapEditorScrollBar, BorderLayout.CENTER);
		
		this.imageBlockScrollBar = new JScrollPane(this.imageBlockPanel,JScrollPane.VERTICAL_SCROLLBAR_ALWAYS,JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS);
		this.imageBlockScrollBar.getViewport().setScrollMode(JViewport.BACKINGSTORE_SCROLL_MODE);
		this.add(this.imageBlockScrollBar, BorderLayout.EAST);
		
	} // End of the constructor with parameters
	
	//*********************************************************************************
	// Methods
	//*********************************************************************************
	/**
	 * Create a image map
	 */
	public Dimension CreateImageMap()
	{
		// Get the image for the tile map.
		try
		{			
			this.tileMapImage = ImageIO.read(new File(this.currentMap.imagePath));	
		}
		catch(IOException ex)
		{
				
		}
		
		// Get the tile width and height from the current map.
		short tileWidth = currentMap.tileWidth;
		short tileHeight = currentMap.tileHeight;
		
		// Get the width and height of the image.
		short imageWidth = (short)this.tileMapImage.getWidth(null);
		short imageHeight = (short)this.tileMapImage.getHeight(null);
		
		// Calculate the total number of tiles.
		short numberOfxTiles = (short)(imageWidth / tileWidth);
		short numberOfyTiles = (short)(imageHeight / tileHeight);
		short numberOfTiles = (short)(numberOfxTiles * numberOfyTiles);

		// Calculate the width of tiles for presentation.
		short numberOfxdTiles = (short)(128 / tileWidth);
		
		// Create a point to return, this point is used to set the width and height of the panel.
		Dimension panelWidthHeight = new Dimension(numberOfxdTiles * (tileWidth + 1), (numberOfTiles / numberOfxdTiles) * (tileHeight + 1));
		
		// Create the list
		this.imageTiles = new ImageTile[numberOfTiles];

		int xdPosition = 0;

		int ydPosition = 0;
		int xdCounter = 0;
		
		int xsPosition = 0;
		int ysPosition = 0;
		int xsCounter = 0;


		// Loop through each tile in the tileMapImage
		for(int i = 0; i < numberOfTiles; i++)
		{
			// Add the image tile
			this.imageTiles[i] = new ImageTile(new Point(xdPosition, ydPosition), 
					new Point(xsPosition, ysPosition));
		
			// Add one to the counters
			xsCounter++;
			xdCounter++;
			
			// Adjust coordinates for the source rectangle
			if(xsCounter >= numberOfxTiles)
			{
				xsPosition = 0;
				ysPosition += tileHeight;
				xsCounter = 0;
			}
			else
			{
				xsPosition += tileWidth;
			}
			
			// Adjust coordinates for the destination rectangle
			if(xdCounter >= numberOfxdTiles)
			{
				xdPosition = 0;
				ydPosition += tileHeight + 1;
				xdCounter = 0;
			}
			else
			{
				xdPosition += tileWidth + 1;
			}
		}
		
		// Return the dimension
		return panelWidthHeight;
		
	} // End of the CreateImageTileList method
	
	/**
	 * Save the current map to a XML file.
	 * @param filePath
	 */
	public void saveToXML(String filePath)
	{
		
		// Declare a output stream
		BufferedWriter outPutStream = null;
		
		try
		{
			// Set the output stream
			outPutStream = new BufferedWriter(new OutputStreamWriter(new FileOutputStream(filePath),"UTF8"));

			Serializer serializer = new Persister(new Format(0, "<?xml version=\"1.0\" encoding=\"utf-8\"?>"));

			serializer.write(this.currentMap, outPutStream);

		}
		catch(PersistenceException exception)
		{
			System.out.print(exception.getMessage());
			JOptionPane.showMessageDialog(this, "The map could not be exported to a XML-file.", "Export to XML file", JOptionPane.ERROR_MESSAGE);
		}
		catch(Exception ex)
		{
			System.out.print(ex.getMessage());
		}
		finally
		{

			// Close the output stream if it not is null.
			if(outPutStream != null)
			{
				try
				{
					outPutStream.close();
				}
				catch (Exception ex)
				{
					
				}
			}
		}
	} // End of the saveToXML method.
	
	/**
	 * Save a map to a object file.
	 * @param filePath
	 */
	public void saveToObjectFile(String filePath)
	{
		ObjectOutputStream oos = null; 
		
		try
		{
			oos = new ObjectOutputStream(new FileOutputStream(filePath));
			oos.writeObject(this.currentMap);		
		}
		catch (Exception ex)
		{
			JOptionPane.showMessageDialog(this, "The map could not be saved.", "Save file", JOptionPane.ERROR_MESSAGE);
		}
		finally
		{
			if(oos != null)
			{
				try
				{
				oos.close();
				}
				catch(Exception ex)
				{
					
				}
			}
		}
		
	} // End of the saveToObjectFile method.
	
	//*********************************************************************************
	// Inner panel classes
	//*********************************************************************************
	/**
	 * A map editor panel
	 */
	public class MapEditorPanel extends JPanel implements MouseListener, MouseMotionListener
	{
		// Class variables
		private static final long serialVersionUID = -3018303250776216030L;

		// Constructor
		public MapEditorPanel(LayoutManager layoutManager)
		{
			super(layoutManager);
			this.setOpaque(true);
			addMouseListener(this);
			addMouseMotionListener(this);
			
		} // End of the constructor
		
		/**
		 * Paint the map editor panel
		 */
		public void paintComponent(Graphics g) 
		{
			super.paintComponent(g);
			
			// Get the rectangle for the view port
			Rectangle viewPort = mapEditorScrollBar.getViewport().getViewRect();
			
			// Create variables for the paint.
			int startTileColumn = (viewPort.x / currentMap.tileWidth);
			int startTileRow = (viewPort.y / currentMap.tileHeight);
			int endTileColumn = (int)(viewPort.getMaxX() / currentMap.tileWidth) + 1;
			int endTileRow = (int)(viewPort.getMaxY() / currentMap.tileWidth) + 1;
			
			if(endTileColumn > currentMap.mapWidth)
				endTileColumn = currentMap.mapWidth;
			if(endTileRow > currentMap.mapHeight)
				endTileRow = currentMap.mapHeight;

			
			for(int y = startTileRow; y < endTileRow; y++)
			{
				for(int x = startTileColumn; x < endTileColumn; x++)
				{
					int xPosition = x * currentMap.tileWidth;
					int yPosition = y * currentMap.tileHeight;
					
					Point imagePoint = imageTiles[currentMap.tiles[y][x]].sImagePoint;
					
					// Draw the image
		 			g.drawImage(tileMapImage, xPosition, yPosition, 
		 					xPosition + currentMap.tileWidth, yPosition + currentMap.tileHeight, 
		 					imagePoint.x, imagePoint.y, imagePoint.x + currentMap.tileWidth, 
		 					imagePoint.y + currentMap.tileHeight, null);
				}
				
			}
			
			/*// Loop through each tile.
			for(int i = startTile; i < currentMap.tiles.length; i++)
 			{
				

				// Create a new tile rectangle
				Rectangle tileRectangle = new Rectangle(xPosition, yPosition, 
						currentMap.tileWidth, currentMap.tileHeight);
				
				// Break from the loop if the endTile has been reached.
				if(i > endTile)
					break;
				
				// Check if the rectangle for the tile includes the mouse point.
				if(viewPort.intersects(tileRectangle))
				{
					
					// Draw the image
		 			g.drawImage(tileMapImage, currentMap.tiles[i].tilePoint.x, currentMap.tiles[i].tilePoint.y, 
		 					currentMap.tiles[i].tilePoint.x + currentMap.tileWidth, currentMap.tiles[i].tilePoint.y + currentMap.tileHeight, 
		 					currentMap.tiles[i].imagePoint.x, currentMap.tiles[i].imagePoint.y, currentMap.tiles[i].imagePoint.x + currentMap.tileWidth, 
		 					currentMap.tiles[i].imagePoint.y + currentMap.tileHeight, null);
				}
 			}*/
			
	    } // End of the paintComponent method.

		@Override
		public void mouseClicked(MouseEvent e) 
		{
			int x = e.getX();
			int y = e.getY();
			
			drawChangedTile(x, y);
			
		} // End of the mouseClicked method

		@Override
		public void mouseEntered(MouseEvent arg0) {}

		@Override
		public void mouseExited(MouseEvent arg0) {}

		@Override
		public void mousePressed(MouseEvent e) {}

		@Override
		public void mouseReleased(MouseEvent arg0) {}

		@Override
		public void mouseDragged(MouseEvent e) 
		{
			int x = e.getX();
			int y = e.getY();
			
			drawChangedTile(x, y);
			
		} // End of the mouseDragged method.

		@Override
		public void mouseMoved(MouseEvent arg0) {}
		
		/**
		 * A helper method to draw a changed tile.
		 * @param x
		 * @param y
		 */
		private void drawChangedTile(int x, int y)
		{
			// Get the rectangle for the view port
			Rectangle viewPort = mapEditorScrollBar.getViewport().getViewRect();
			
			// Create variables for the paint.
			int startTileColumn = (viewPort.x / currentMap.tileWidth);
			int startTileRow = (viewPort.y / currentMap.tileHeight);
			int endTileColumn = (int)(viewPort.getMaxX() / currentMap.tileWidth);
			int endTileRow = (int)(viewPort.getMaxY() / currentMap.tileWidth);

			
			for(int r = startTileRow; r < endTileRow; r++)
			{
				for(int c = startTileColumn; c < endTileColumn; c++)
				{
					int xPosition = c * currentMap.tileWidth;
					int yPosition = r * currentMap.tileHeight;
					
					// Create a new rectangle
					Rectangle tileRectangle = new Rectangle(xPosition, yPosition, 
							currentMap.tileWidth, currentMap.tileHeight);
						
					// Check if the rectangle for the tile includes the mouse point.
					if(tileRectangle.contains(x, y))
					{
						currentMap.tiles[r][c] = selectedImage;
						
						this.repaint();
						break;
					}
				}
				
			}
			
			/*// Get the view port rectangle.
			Rectangle viewPort = mapEditorScrollBar.getViewport().getViewRect();
			
			int columnNumber = (viewPort.x / currentMap.tileWidth);
			int rowNumber = (viewPort.y / currentMap.tileHeight);
			int startTile = (rowNumber * currentMap.mapWidth) + columnNumber;
			int endTileColumn = (int)(viewPort.getMaxX() / currentMap.tileWidth);
			int endTileRow = (int)(viewPort.getMaxY() / currentMap.tileHeight);
			int endTile = (endTileRow * currentMap.mapWidth) + endTileColumn;
	
			// Loop through each tile.
			for(int i = startTile; i < currentMap.tiles.length; i++)
 			{
				// Create a new rectangle
				Rectangle tileRectangle = new Rectangle(currentMap.tiles[i].tilePoint.x, currentMap.tiles[i].tilePoint.y, 
						currentMap.tileWidth, currentMap.tileHeight);
				
				// Break from the loop if the endTile has been reached.
				if(i > endTile)
					break;
				
				// Check if the rectangle for the tile includes the mouse point.
				if(tileRectangle.contains(x, y))
				{
					currentMap.tiles[i].imagePoint = selectedImage;
					
					this.repaint();
					break;
				}
 			}*/
			
		} // End of the drawTile method.
		
	} // End of the MapEditorPanel class
	
	/**
	 * A panel for a tilemap image
	 * @author Fredrik Stigsson
	 *
	 */
	public class ImageTilePanel extends JPanel implements MouseListener
	{

		private static final long serialVersionUID = 6558962207035959399L;
		
		public ImageTilePanel(LayoutManager layoutManager)
		{
			super(layoutManager);
			this.setOpaque(true);
			addMouseListener(this);
		}
		
		/**
		 * Paint the panel
		 */
		public void paintComponent(Graphics g) 
		{
			super.paintComponent(g); 
	         
			for(ImageTile imageTile : imageTiles)
 			{	
 				// Draw the image
	 			g.drawImage(tileMapImage, imageTile.dImagePoint.x, imageTile.dImagePoint.y, imageTile.dImagePoint.x + currentMap.tileWidth, 
	 					imageTile.dImagePoint.y + currentMap.tileHeight, imageTile.sImagePoint.x, imageTile.sImagePoint.y, imageTile.sImagePoint.x + currentMap.tileWidth, 
	 					imageTile.sImagePoint.y + currentMap.tileHeight, null);

	 		}
			
	    } // End of the paintComponent method.

		@Override
		public void mouseClicked(MouseEvent e) 
		{
			int x = e.getX();
			int y = e.getY();
			
			for(int i = 0; i < imageTiles.length; i++)
			{
				// Create a rectangle
				Rectangle imageRectangle = new Rectangle(imageTiles[i].dImagePoint.x, imageTiles[i].dImagePoint.y, 
						currentMap.tileWidth, currentMap.tileHeight);
				
				if(imageRectangle.contains(x, y))
				{
					selectedImage = (short)i;
					break;
				}
			}		
		}

		@Override
		public void mouseEntered(MouseEvent arg0) {}

		@Override
		public void mouseExited(MouseEvent arg0) {}

		@Override
		public void mousePressed(MouseEvent arg0) {}

		@Override
		public void mouseReleased(MouseEvent arg0) {}
		
	} // End of the ImageTilePanel class
	
} // End of the outer class
