package main;

import java.awt.Component;
import java.awt.event.*;
import javax.swing.*;

import org.simpleframework.xml.*;
import org.simpleframework.xml.core.*;
import org.simpleframework.xml.stream.Format;

import internalWindows.*;
import mapObjects.*;

import java.io.*;

/**
 * This class represent the main window for this program. It inherits from JFrame and
 * implements the ActionListener interface.
 * @author Fredrik Stigsson
 *
 */
public class MainFrame extends JFrame implements ActionListener
{
	
	//*********************************************************************************
	// Class variables
	//*********************************************************************************
	private static final long serialVersionUID = 3503909200415714774L;
	
	//*********************************************************************************
	// Instance variables
	//*********************************************************************************
	private JDesktopPane desktopPane;
	
	//*********************************************************************************
	// Constructors
	//*********************************************************************************
	/**
	 * Create a new main window with a default size.
	 */
	public MainFrame()
	{
		// Call the constructor in the base class and set the name of the frame.
		super("Game map editor 2D");
		
		// Initialize instance variables
		this.desktopPane = new JDesktopPane();
		this.setContentPane(desktopPane);

		// Maximize the window.
		this.setExtendedState(this.getExtendedState()|JFrame.MAXIMIZED_BOTH);
		
		// Make dragging a little faster but perhaps uglier.
		desktopPane.setDragMode(JDesktopPane.OUTLINE_DRAG_MODE);
        
		// Set the default close operation.
		this.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		
		// Build the menu.
		buildMenu();
			
	} // End of the default constructor.
	
	//*********************************************************************************
	// Methods
	//*********************************************************************************
	/**
	 * A helper method to create and add a menu.
	 */
	private void buildMenu()
	{
		// Create a new menu bar.
		JMenuBar menuBar = new JMenuBar();
		
		// Create menus.
		JMenu file = new JMenu("File");
		
		// Create menu items.
		JMenuItem menuItemNew = new JMenuItem("New map");
		JMenuItem menuItemOpen = new JMenuItem("Open map");
		JMenuItem menuItemSave = new JMenuItem("Save map");
		JMenuItem menuItemExport = new JMenuItem("Export map to xml");
		JMenuItem menuItemImport = new JMenuItem("Import map from xml");
		JMenuItem menuItemExit = new JMenuItem("Exit");
		
		// Add listeners for menu items.
		menuItemNew.addActionListener(this);
		menuItemOpen.addActionListener(this);
		menuItemSave.addActionListener(this);
		menuItemExport.addActionListener(this);
		menuItemImport.addActionListener(this);
		menuItemExit.addActionListener(this);

		// Add menus/menu items to menus.
		file.add(menuItemNew);
		file.add(menuItemOpen);
		file.add(menuItemSave);
		file.add(menuItemExport);
		file.add(menuItemImport);
		file.add(menuItemExit);

		// Add menus to the menu bar
		menuBar.add(file);

		// Add the menu bar for the frame.
		this.setJMenuBar(menuBar);
		
	} // End of the buildMenu method.
	
	//*********************************************************************************
	// Events
	//*********************************************************************************
	/**
	 * Handles click events for menu items.
	 * @param An ActionEvent denoted as e
	 */
	public void actionPerformed(ActionEvent e)
	{
		// Get the action command.
		String actionCommand = e.getActionCommand();
		
		// Do actions based on the action command.
		if(actionCommand.equalsIgnoreCase("New map"))
		{
			// Create a new internal frame
			CreateNewMapFrame internalFrame = new CreateNewMapFrame(this.getContentPane());
			
			// Set the frame as visible before it is added to the desktopPane.
			internalFrame.setVisible(true);
			
			// Add the frame to the desktop pane, to the front (0).
			this.desktopPane.add(internalFrame, 0);
			
		}
		else if (actionCommand.equalsIgnoreCase("Open map"))
		{
			
			// Create a new file chooser
			JFileChooser chooseFile = new JFileChooser(".");
			
			// Get the choice option as an int.
			int choice = chooseFile.showOpenDialog(this);
			
			// Read the file if the user has approved this.
			if (choice == JFileChooser.APPROVE_OPTION)
			{
				// Close all open map editor frames
				Component[] internalFrames = this.desktopPane.getComponents();
				for(Component comp : internalFrames)
				{
					if(comp instanceof MapEditorFrame)
					{
						MapEditorFrame mapFrame = (MapEditorFrame)comp;
						mapFrame.dispose();
					}
				}
				
				ObjectInputStream inputStream = null;
				
				try
				{
					// Create a new objectInputStream
					inputStream = new ObjectInputStream(new FileInputStream(chooseFile.getSelectedFile().toString()));
					
					// Read the object from the file and cast it to a Map2D object
					Map2D currentMap = (Map2D)inputStream.readObject();
					
					// Create a new map editor frame
					MapEditorFrame internalFrame = new MapEditorFrame(currentMap, this.desktopPane.getSize());
					
					// Add the internal frame to the desktop pane of the main frame
					this.desktopPane.add(internalFrame);
					
					// Set the frame as visible before it is added to the desktopPane.
					internalFrame.setVisible(true);
					
				}
				catch (Exception exception)
				{
					// Show a error message
					JOptionPane.showMessageDialog(this, "The file could not be opened", "Open file", JOptionPane.ERROR_MESSAGE);
				}
				finally
				{
					// Close the inputStream
					if(inputStream != null)
					{
						try
						{
						inputStream.close();
						}
						catch(Exception ex)
						{
							
						}
					}
				}			
			}
		}
		else if (actionCommand.equalsIgnoreCase("Save map"))
		{
			// Create a new file chooser
			JFileChooser chooseFile = new JFileChooser(".");
			
			// Set the default filename for the file.
			chooseFile.setSelectedFile(new File("MapProject.sav"));
			
			// Get the choice option as an int.
			int choice = chooseFile.showSaveDialog(this);
			
			// Get the map editor frame
			MapEditorFrame mapFrame = null;
			Component[] internalFrames = this.desktopPane.getComponents();
			for(Component comp : internalFrames)
			{
				if(comp instanceof MapEditorFrame)
				{
					mapFrame = (MapEditorFrame)comp;
				}
			}
			
			// Save the current map to a object file
			if (choice == JFileChooser.APPROVE_OPTION)
			{
				mapFrame.saveToObjectFile(chooseFile.getSelectedFile().toString());
			}	
		}
		else if (actionCommand.equalsIgnoreCase("Export map to xml"))
		{
			// Create a new file chooser
			JFileChooser chooseFile = new JFileChooser(".");
			
			// Set the default filename for the file.
			chooseFile.setSelectedFile(new File("Map.xml"));
			
			// Get the choice option as an int.
			int choice = chooseFile.showSaveDialog(this);
			
			// Get the map editor frame
			MapEditorFrame mapFrame = null;
			Component[] internalFrames = this.desktopPane.getComponents();
			for(Component comp : internalFrames)
			{
				if(comp instanceof MapEditorFrame)
				{
					mapFrame = (MapEditorFrame)comp;
				}
			}
			
			// Save the current map to a XML file.
			if (choice == JFileChooser.APPROVE_OPTION)
			{
				mapFrame.saveToXML(chooseFile.getSelectedFile().toString());
			}
			
		}
		else if (actionCommand.equalsIgnoreCase("Import map from xml"))
		{
			
			// Create a new file chooser
			JFileChooser chooseFile = new JFileChooser(".");
			
			// Get the choice option as an int.
			int choice = chooseFile.showOpenDialog(this);
			
			// Read the file if the user has approved this.
			if (choice == JFileChooser.APPROVE_OPTION)
			{
				// Close all open map editor frames
				Component[] internalFrames = this.desktopPane.getComponents();
				for(Component comp : internalFrames)
				{
					if(comp instanceof MapEditorFrame)
					{
						MapEditorFrame mapFrame = (MapEditorFrame)comp;
						mapFrame.dispose();
					}
				}
				
				// Declare a input stream
				BufferedReader inputReader = null;
				
				try
				{
					// Set the input stream
					inputReader = new BufferedReader(new InputStreamReader(new FileInputStream(chooseFile.getSelectedFile().toString()),"UTF8"));
					
					// Create a new Serializer object
					Serializer serializer = new Persister(new Format(0, "<?xml version=\"1.0\" encoding=\"utf-8\"?>"));

					// Read the object from the xml file and cast it to a Map2D object
					Map2D currentMap = serializer.read(Map2D.class, inputReader);

					// Create a new map editor frame
					MapEditorFrame internalFrame = new MapEditorFrame(currentMap, this.desktopPane.getSize());
					
					// Add the internal frame to the desktop pane of the main frame
					this.desktopPane.add(internalFrame);
					
					// Set the frame as visible before it is added to the desktopPane.
					internalFrame.setVisible(true);
					
				}
				catch (Exception exception)
				{
					// Show a error message
					JOptionPane.showMessageDialog(this, "The file could not be opened", "Open file", JOptionPane.ERROR_MESSAGE);
				}
				finally
				{
					// Close the inputReader
					if(inputReader != null)
					{
						try
						{
							inputReader.close();
						}
						catch(Exception ex)
						{
							
						}
					}
				}			
			}
		}
		else if (actionCommand.equalsIgnoreCase("Exit"))
		{
			System.exit(0);
		}
		
	} // End of actionPerformed
	
} // End of the class
